#!/usr/bin/env sh
# shellcheck source=onedatify/onedatify

support_usage() {
s=${0##*/}
margin=$(printf "%-${#s}s" " ")
cat <<EOF
Support a space with a storage backend and enabled import of existing data residing on the storage backend into the space.

${0##*/} support --token <space support token> --size <size in bytes> { --storage-id <id> | --storage-name }
${margin}       { --import [ --import-maxdepth <scan depth> ] [ --import-sync-acl ]  }
${margin}       { --update [ --update-maxdepth <scan depth> ] [ --update-scan-interval <seconds> ] [ --update-sync-acl ]
${margin}       [ --update-maxdepth <scan depth> ] [ --update-writeOnce ] [ --update-delete ] [ --update-sync-acl ] }

Options:
  -t, --token              the token for space support.
  -s, --size               the storage space size that provider is willing to assign to the space. 
                           By default the value is in bytes. Units B, KB, GB, TB, PT are also supported et. 10GB.
  -i, --storage-id         the ID of the storage backend where the space data should be stored.
  -n, --storage-name       alternatively to --storage-id, you can supply the storage backend name
  --import                 enable storage import. Allows to import data from storage backend to space without need for copying the data.
  --import-maxdepth        maximum depth of filesystem tree that will be traversed during storage synchronization (default: 65535). Implies --import.
  --import-sync-acl        enable synchronization of NFSv4 ACLs (default: false). Implies --import.
  --update                 enable storage update scan. Ensures that all changes on the storage backend will be reflected in space. Implies --update. Implies --update.
  --update-maxdepth        maximum depth of filesystem tree that will be traversed during update scan (default: 65535). Implies --update.
  --update-scan-interval   time [s] of subsequent scans, counted from end of one scan till beginning of the following (default: 10 seconds). Implies --update.
  --update-writeOnce       should synchronized storage be treated as immutable, only creations and deletions of files on the storage backend will be detected (default: false). Implies --update.
  --update-disable-delete  disable detection of deleted files (default: false). Implies --update.
  --update-sync-acl        enable updating of NFSv4 ACLs (default: false). Implies --update.

Examples:
Support space with storage backend named "posix" with 1GB (1024^3):
${0##*/} support -t MDA... --s 1073741824 -i fdc2afc0d617f6f541d20ea0e0e8cf14

Same example, just once scan all data on this storage backend and make it visible in the space:
${0##*/} support -t MDA... --s 1073741824 -i fdc2afc0d617f6f541d20ea0e0e8cf14 --import

In case of a big dataset where the data does not change frequently, you want to rescan it every 12 hours,
to make changes on the storage backend visible in the space:
${0##*/} support -t MDA... --s 1073741824 -i fdc2afc0d617f6f541d20ea0e0e8cf14 --import --update-scan-interval 43200
EOF
}

support_main() {

  if [ $(( $# )) -eq 0 ]; then
    support_usage
    exit 0
  fi

# mandatory
support_name=""
support_token=""
support_size=""
support_storageId=""

# mandatory for import
# the values on the right in comments are the api defaults
support_storageImport_strategy=0

# optional for import
support_storageImport_maxDepth=""
support_storageImport_syncAcl=0

# mandatory for update
support_storageUpdate_strategy=0

# optional for update
support_storageUpdate_maxDepth="" 
support_storageUpdate_scanInterval=""
support_storageUpdate_writeOnce=0
support_storageUpdate_deleteEnable=1
support_storageUpdate_syncAcl=0

support_interactive_flag=0
  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              support_usage
              exit 0
              ;;
          interactive)
              support_interactive_flag=1
              ;;
          -t|--token)
              support_token="$2"
              shift
              ;;
          -s|--size)
              support_size="$2"
              shift
              ;;
          -i|--storage-id)
              support_storageId="$2"
              shift
              ;;
          -n|--storage-name)
              support_storage_name="$2"
              shift
              ;;
          --import)
              support_storageImport_strategy=1
              ;;
          --import-maxdepth)
              support_storageImport_maxDepth="$2"
              shift
              ;;
          --import-sync-acl)
              support_storageImport_syncAcl=1
              ;;
          --update)
              support_storageUpdate_strategy=1
              ;;
          --update-maxdepth)
              support_storageUpdate_maxDepth="$2"
              shift
              ;;
          --update-scan-interval)
              support_storageUpdate_scanInterval="$2"
              shift
              ;;
          --update-writeOnce)
              support_storageUpdate_writeOnce=1
              ;;
          --update-disable-delete)
              support_storageUpdate_deleteEnable=0
              ;;
          --update-sync-acl)
              support_storageUpdate_syncAcl=1
              ;;
          -?*|*)
              printf 'WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  . ${onedatify_config_dir}/config.sh
  if ! config_main validate 1; then
    echo "ERROR. Configuration is not complete or contains errors. Please run onedatify config to fix it"
    exit 1
  fi

  # TODO validate mount point
  if [ "$support_token" = "" ]; then
    echo "Missing mandatory --token parameter. Please see --help."
    exit 1
  fi

  if [ "$support_size" = "" ]; then
    echo "Missing mandatory --size parameter. Please see --help."
    exit 1
  elif ! validate_disk_size "$support_size"; then
    echo "$validate_msg_error"
    exit 1
  else 
    support_size=$(convert_disk_size "$support_size")
  fi

  # TODO validate mount point
  if [ "$support_storageId" = "" ] && [ "$support_storage_name" = "" ]; then
    echo "Missing mandatory --storage-id parameter. Please see --help."
    exit 1
  elif [ "$support_storageId" = "" ] && [ "$support_storage_name" != "" ]; then
    . "${onedatify_config_dir}/storage.sh"
    support_storageId=$(storage_get -s name="$support_storage_name" -f id -v)

    if [ "$(echo "$support_storageId" | grep -c .)" -gt "1" ] ; then
      echo "Error: multiple storage backends with the name $support_storage_name are registered:"
      echo "$support_storageId"
      echo "please use --storage-id to identify specific storage."
      exit 1
    fi
  fi

  # TODO validate this
  # this is optional
  if  [ "$support_storageImport_maxDepth" != "" ]; then
    support_storageImport_maxDepth_json=",\"maxDepth\":\"$support_storageImport_maxDepth\""
    support_storageImport_strategy=1
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageImport_syncAcl" = "1" ]; then
    support_storageImport_syncAcl_json=",\"syncAcl\":\"true\""
    support_storageImport_strategy=1
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageImport_strategy" = "1" ]; then
    support_storageImport_strategy="simple_scan"
support_storageImport_strategy_json=$(cat <<EOF
    ,"storageImport": {
      "strategy": "$support_storageImport_strategy"
$support_storageImport_maxDepth_json
$support_storageImport_syncAcl_json
    }
EOF
)
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageUpdate_maxDepth" != "" ]; then
    support_storageUpdate_maxDepth_json=",\"maxDepth\":\"$support_storageUpdate_maxDepth\""
    #support_storageUpdate_strategy=1
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageUpdate_scanInterval" != "" ]; then
    support_storageUpdate_scanInterval_json=",\"scanInterval\":\"$support_storageUpdate_scanInterval\""
    #support_storageUpdate_strategy=1
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageUpdate_writeOnce" = "1" ]; then
    support_storageUpdate_writeOnce_json=",\"writeOnce\":\"true\""
    #support_storageUpdate_strategy=1
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageUpdate_deleteEnable" = "1" ]; then
    support_storageUpdate_deleteEnable_json=",\"deleteEnable\":\"true\""
    #support_storageUpdate_strategy=1
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageUpdate_syncAcl" = "1" ]; then
    support_storageUpdate_syncAcl_json=",\"syncAcl\":\"true\""
    #support_storageUpdate_strategy=1
  fi

  if [ "$support_interactive_flag" = "1" ]; then
    . "${onedatify_config_dir}/storage.sh"
    support_interactive_storage_id=$(storage_get -v -f id | head -1)
    support_interactive_storage_name=$(storage_get -v -s id="$support_interactive_storage_id"  -f name| head -1)
    support_interactive_storage_type=$(storage_get -v -s id="$support_interactive_storage_id"  -f type| head -1)

    support_size_default=$support_size
    support_size=''
    while [ "$support_size" = "" ] || ! validate_disk_size "$support_size" ; do
      display_support_size_default=$(convert_back_disk_size "$support_size_default")
      _message "The size of space support (you can use B, KiB, MiB, up to EiB, 1KiB = 1024B, eg. 1GiB) (auto-detected: ${display_support_size_default}): "
      read -r support_size </dev/tty
      [  "$support_size" = "" ] && support_size=$support_size_default
      if [ "$support_size" = "" ] || ! validate_disk_size "$support_size" ; then
        message_imp "$validate_msg_error"
      else
        support_size=$(convert_disk_size "$support_size")
      fi
    done

    if [ "$support_storageUpdate_strategy" = "1" ]; then
      if [ "$support_storageUpdate_strategy" = "1" ]; then
          support_storageUpdate_strategy_default=y
      else
          support_storageUpdate_strategy_default=n
      fi
      support_storageUpdate_strategy=""
      while [ "$support_storageUpdate_strategy" != "n" ] && [ "$support_storageUpdate_strategy" != "y" ] ; do
        _message "Do you want to regularly scan the storage for any changes in files? (y/n, default: $support_storageUpdate_strategy_default): "
        read -r support_storageUpdate_strategy </dev/tty
        [  "$support_storageUpdate_strategy" = "" ] && support_storageUpdate_strategy=$support_storageUpdate_strategy_default
      done
    fi
    if [ "$support_storageUpdate_strategy" = "y" ]; then
        support_storageUpdate_strategy=1
    else
        support_storageUpdate_strategy=0
    fi
    echo ""
    message "Supporting a space with $support_size [B] of storage $support_interactive_storage_name on oneprovider $config_fqdn"

    support_storageId="$support_interactive_storage_id"
  fi

  # TODO validate this
  # this is optional
  if [ "$support_storageUpdate_strategy" = "1" ]; then
    support_storageUpdate_strategy="simple_scan"
support_storageUpdate_strategy_json=$(cat <<EOF
    ,"storageUpdate": {
      "strategy": "$support_storageUpdate_strategy"
$support_storageUpdate_maxDepth_json
$support_storageUpdate_scanInterval_json
$support_storageUpdate_writeOnce_json
$support_storageUpdate_deleteEnable_json
$support_storageUpdate_syncAcl_json
    }
EOF
)
  fi

storage_support_json=$(cat <<EOF
  {
    "token": "$support_token",
    "size": "$support_size",
    "storageId": "$support_storageId"
$support_name_json
$support_storageImport_strategy_json
$support_storageUpdate_strategy_json
  }
EOF
)

  #if storage_add_posix_was_oneprovider_running; then
  onedatify_start

  . "${onedatify_config_dir}/install.sh"
  _message "Checking Oneprovider readiness." ; install_provider_ready_check ; echo ""
  #fi

  storage_support_script=$(cat <<EEOF
#!/usr/bin/env bash
cat <<EOF >/tmp/json
${storage_support_json}
EOF
cat /tmp/json | onepanel-rest-cli supportSpace -
EEOF
  )
  storage_support_id_response=$(echo "$storage_support_script" | $sh_c "docker run \
    -i \
    --network=host \
    -e TERM=xterm \
    -e ONEPANEL_HOST=\"https://localhost:9443\" \
    -e ONEPROVIDER_HOST=\"https://localhost\" \
    -e ONEPANEL_BASIC_AUTH=\"onepanel:$config_admin_password\" \
    --rm \
    --entrypoint bash \
    \"$rest_cli_docker_image\" -c \
    'echo \"-k\" > ~/.curlrc ; cat > script.sh ; chmod +x script.sh ; ./script.sh'"
  )
  
  if [ "$support_interactive_flag" = "1" ]; then
    tput sgr0 || tput me s
    if echo "$storage_support_id_response" | grep '^{"id":"' --quiet ; then
        tput setaf 7 ; tput bold || tput md
    cat <<EOF

Your storage was successfully supported and exposed.
Depeding on the amount of exposed data it might take some time to scan it. Please be patient.

To configure your Oneprovider please visit:
https://$config_fqdn:9443

In order to access your data please visit:
https://$config_zone_fqdn

Generated password to access the Onepanel emergency account is: 
${config_admin_password}

EOF
        tput sgr0 || tput me
    else
        tput setaf 1; tput bold || tput md
    cat <<EOF

We were unable to support your space!

Error:
$storage_support_id_response

You can continue to configure your Oneprovider and support your space manually via Onepanel emergency account.

Please visit:
https://$config_fqdn:9443

and use the passphrase:
${config_admin_password}
to access the Onepanel emergency account.


EOF
        tput sgr0 || tput me
    fi
  else 
    echo "$storage_support_id_response"
  fi
}

