#!/usr/bin/env sh
# shellcheck source=onedatify/onedatify

upgrade_backup() {
  backup_path="$1/$(date +%Y.%m.%d)"

  echo "----------------------------------------------------------------------"
  echo "Backing up Oneprovider persistence and config into $backup_path"
  echo "----------------------------------------------------------------------"

  sudo mkdir -p $backup_path
  sudo rsync -asv --info=progress2 /opt/onedata/onedatify $backup_path

  echo "----------------------------------------------------------------------"
  echo "Backup complete"
  echo "----------------------------------------------------------------------"
}

upgrade_retag() {

  # Divide new docker image tag into components
  upgrade_new_full_name="$2"
  upgrade_new_tag=${upgrade_new_full_name##*:}
  upgrade_retag_tmp=${upgrade_new_full_name%$upgrade_new_tag}
  upgrade_retag_tmp=${upgrade_retag_tmp%:}
  upgrade_new_image=${upgrade_retag_tmp##*/}
  upgrade_retag_tmp=${upgrade_retag_tmp%$upgrade_new_image}
  upgrade_retag_tmp=${upgrade_retag_tmp%/}
  upgrade_new_organization=${upgrade_retag_tmp##*/}
  upgrade_retag_tmp=${upgrade_retag_tmp%$upgrade_new_organization}
  upgrade_retag_tmp=${upgrade_retag_tmp%/}
  upgrade_new_private_repo=$upgrade_retag_tmp

  # Divide old docker image tag into components
  upgrade_old_full_name="$1" #=$(sed -n "s#^[[:space:]]*image: *\([^[:space:]]*\)#\1#gp" "$upgrade_retag_config_path")
  upgrade_old_tag=${upgrade_old_full_name##*:}
  upgrade_retag_tmp=${upgrade_old_full_name%$upgrade_old_tag}
  upgrade_retag_tmp=${upgrade_retag_tmp%:}
  upgrade_old_image=${upgrade_retag_tmp##*/}
  upgrade_retag_tmp=${upgrade_retag_tmp%$upgrade_old_image}
  upgrade_retag_tmp=${upgrade_retag_tmp%/}
  upgrade_old_organization=${upgrade_retag_tmp##*/}
  upgrade_retag_tmp=${upgrade_retag_tmp%$upgrade_old_organization}
  upgrade_retag_tmp=${upgrade_retag_tmp%/}
  upgrade_old_private_repo=$upgrade_retag_tmp

  # If new docker tag is missing error
  if [ "$upgrade_new_tag" = "" ]; then
    echo "Error: cannot parse docker image: $upgrade_new_full_name"
    exit 1
  fi

  # if user only supplied tag. as in "latest", we substitute this part only
  if [ "$upgrade_new_image" = "" ] && [ "$upgrade_new_organization" = "" ] && [ "$upgrade_new_private_repo" = "" ]; then
    upgrade_old_full_name="$upgrade_old_organization/$upgrade_old_image:$upgrade_new_tag"
    [ "$upgrade_old_private_repo" != "" ] && upgrade_old_full_name="$upgrade_old_private_repo/$upgrade_old_full_name"
  # if user gave the full image then we use full
  elif [ "$upgrade_new_image" != "" ] && [ "$upgrade_new_organization" != "" ]; then
    upgrade_old_full_name="$upgrade_new_organization/$upgrade_new_image:$upgrade_new_tag"
    [ "$upgrade_new_private_repo" != "" ] && upgrade_old_full_name="$upgrade_new_private_repo/$upgrade_old_full_name"
  # every other combination is wrong
  else
    echo "Error: cannot parse docker image: $upgrade_new_full_name"
    exit 1
  fi

  config_version="$upgrade_old_full_name"
  config_save "version" "$config_file_path"
  #TODO backup existing compose file
  #TODO save exiting tag as as backup 
  # sed -i "s#\(^[[:space:]]*image: *\)[^[:space:]]*#\1$upgrade_old_full_name#g" "$upgrade_retag_config_path"
}

upgrade_main() {

  if [ $(( $# )) -eq 0 ]; then
    upgrade_usage
    exit 0
  fi

  upgrade_no_backup=""
  upgrade_backup_path=""
  upgrade_oneprovider_version=""

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              upgrade_usage
              exit 0
              ;;
          -n|--no-backup)
              upgrade_no_backup=1
              ;;
          -p|--backup-path)
              upgrade_backup_path="$2"
              shift
              ;;
          -v|--version)
              upgrade_oneprovider_version="$2"
              shift
              ;;
          -?*|*)
              printf 'WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  if [ "$upgrade_oneprovider_version" = "" ]; then
    echo "Error: please provide oneprovider upgrade version. See -h for details."
    exit 1
  fi

  . ${onedatify_config_dir}/config.sh
  if ! config_main validate 1; then
    echo "ERROR. Configuration is not complete or contains errors. Please run onedatify config to fix it"
    exit 1
  fi

  onedatify_stop

  if [ "$upgrade_no_backup" = "" ]; then
    if ! command_exists rsync; then
      echo "Cannot find rsync. Please install it and rerun the script."
      exit 1
    fi

    if [ "$upgrade_backup_path" = "" ]; then
      upgrade_backup_path="${onedatify_config_dir}/../oneprovider-backups"
    fi
    upgrade_backup "$upgrade_backup_path"
  fi

  upgrade_retag "$config_version" "$upgrade_oneprovider_version"

  onedatify_start
}

upgrade_usage() {
cat <<EOF
onedatify upgrade --version <oneprovider docker tag> [ --backup-path <path> ]

  -n,--no-backup      do not back up the persistence before upgrade - NOT RECOMMENDED
  -p,--backup-path    backup destination, default /opt/onedata/oneprovider-backup
  -v,--version        new oneprovider docker image tag 

Example:
Upgrade Oneprovider service with backup disabled:
~$ onedatify install -n -v onedata/oneprovider:21.02.3

EOF
}
